/*
 * Ben NanoNote LCD initialisation, based on uboot-xburst and xburst-tools.
 *
 * Copyright (C) 2001-2002 Wolfgang Denk, DENX Software Engineering, <wd@denx.de>
 * Copyright (C) 2015, 2016 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef CONFIG_CPU_JZ4730_MINIPC
#include "minipc_claa070vc01.h"
#include "minipc.h"
#else
#include "nanonote_gpm940b0.h"
#include "nanonote.h"
#endif

#include "xburst_types.h"
#include "jzlcd.h"
#include "sdram.h"
#include "board.h"

extern vidinfo_t panel_info;
static unsigned long lcd_base;
unsigned long irq_counter = 0;

void test_pixel(unsigned short h, unsigned short v)
{
	unsigned short v_max  = panel_info.vl_row;
	unsigned short h_max  = panel_info.vl_col;
	unsigned long *pix = (unsigned long *)lcd_base + v * h_max + h;

	/* NOTE: Code assumes 32 bits/pixel. */
#ifdef NORMAL
	*pix = (
		(((255 * (h_max - h)) / (h_max - 1)) << 16) +
		((((255 * h) / (h_max - 1) + (255 * (v_max - v)) / (v_max - 1)) / 2) << 8) +
		((255 * v) / (v_max - 1))
		);
#elif HANDLER
	unsigned short offset, bit;
	bit = 31 - (32 * h / (h_max - 1));
	offset = v / 4;
	*pix = ((*((unsigned long *) 0x80000200 + offset) & (1 << bit)) >> bit) ? 0xffffffff : 0;
#else
	unsigned short bit;
	volatile unsigned int cp0_register;
	asm volatile(
		"mfc0 %0, $12\n"
		"nop"
		: "=r"(cp0_register)
		);
	bit = 31 - (32 * h / (h_max - 1));
	*pix = (((cp0_register & (1 << bit)) >> bit) ? 0x007f0000 : 0x0000001f) * ((bit % 2) ? 2 : 1);
	/* *pix = (((irq_counter & (1 << bit)) >> bit) ? 0x007f0000 : 0x0000001f) * ((bit % 2) ? 2 : 1); */
#endif
}

void clear_pixel(unsigned short h, unsigned short v)
{
	unsigned short h_max  = panel_info.vl_col;
	unsigned long *pix = (unsigned long *)lcd_base + v * h_max + h;

	*pix = 0;
}

void test_pattern(void *lcd_base)
{
	unsigned short v_max  = panel_info.vl_row;
	unsigned short h_max  = panel_info.vl_col;
	unsigned short v, h;

	for (v = 0; v < v_max; v += 1) {
		for (h = 0; h < h_max; h += 1) {
			test_pixel(h, v);
		}
	}
}

void lcd_clear(unsigned long lcd_base)
{
	unsigned short v_max  = panel_info.vl_row;
	unsigned short h_max  = panel_info.vl_col;
	unsigned short v, h;
	unsigned long *pix = (unsigned long *)lcd_base;

	for (v = 0; v < v_max; v += 1) {
		for (h = 0; h < h_max; h += 1) {
			*pix++ = 0;
		}
	}
}

/* LCD initialisation. */

void lcd_init(void)
{
	__lcd_display_pin_init();
	__lcd_display_on();

	lcd_base = lcd_ctrl_init();
	lcd_clear(lcd_base);
	lcd_enable();
}
